@@ -313,9 +313,9 @@ class BaseWizard(Logger):
                     continue
                 # see if plugin recognizes 'scanned_devices'
                 try:
-                    # FIXME: side-effect: unpaired_device_info sets client.handler
-                    device_infos = devmgr.unpaired_device_infos(None, plugin, devices=scanned_devices,
-                                                                include_failing_clients=True)
+                    # FIXME: side-effect: this sets client.handler
+                    device_infos = devmgr.list_pairable_device_infos(
+                        handler=None, plugin=plugin, devices=scanned_devices, include_failing_clients=True)
                 except HardwarePluginLibraryUnavailable as e:
                     failed_getting_device_infos(name, e)
                     continue
@@ -351,7 +351,7 @@ class BaseWizard(Logger):
             state = _("initialized") if info.initialized else _("wiped")
             label = info.label or _("An unnamed {}").format(name)
             try: transport_str = info.device.transport_ui_string[:20]
-            except: transport_str = 'unknown transport'
+            except Exception: transport_str = 'unknown transport'
             descr = f"{label} [{info.model_name or name}, {state}, {transport_str}]"
             choices.append(((name, info), descr))
         msg = _('Select a device') + ':'
@@ -622,7 +622,7 @@ class BaseWizard(Logger):
                 password = k.get_password_for_storage_encryption()
             except UserCancelled:
                 devmgr = self.plugins.device_manager
-                devmgr.unpair_xpub(k.xpub)
+                devmgr.unpair_pairing_code(k.pairing_code())
                 raise ChooseHwDeviceAgain()
             except BaseException as e:
                 self.logger.exception('')
@@ -719,7 +719,11 @@ class BaseWizard(Logger):
 
     def confirm_seed(self, seed, passphrase):
         f = lambda x: self.confirm_passphrase(seed, passphrase)
-        self.confirm_seed_dialog(run_next=f, seed=seed if self.config.get('debug_seed') else '', test=lambda x: x==seed)
+        self.confirm_seed_dialog(
+            run_next=f,
+            seed=seed if self.config.get('debug_seed') else '',
+            test=lambda x: mnemonic.is_matching_seed(seed=seed, seed_again=x),
+        )
 
     def confirm_passphrase(self, seed, passphrase):
         f = lambda x: self.run('create_keystore', seed, x)
