@@ -14,7 +14,7 @@ from kivy.properties import StringProperty
 
 from electrum.invoices import (PR_DEFAULT_EXPIRATION_WHEN_CREATING,
                                PR_PAID, PR_UNKNOWN, PR_EXPIRED, PR_INFLIGHT,
-                               pr_expiration_values, Invoice)
+                               pr_expiration_values, Invoice, Request)
 from electrum import bitcoin, constants
 from electrum import lnutil
 from electrum.transaction import tx_from_any, PartialTxOutput
@@ -126,9 +126,12 @@ class HistoryScreen(CScreen):
             fee_text = '' if fee is None else 'fee: %d sat'%fee
         else:
             tx_hash = tx_item['txid']
-            tx_mined_info = TxMinedInfo(height=tx_item['height'],
-                                        conf=tx_item['confirmations'],
-                                        timestamp=tx_item['timestamp'])
+            tx_mined_info = TxMinedInfo(
+                height=tx_item['height'],
+                conf=tx_item['confirmations'],
+                timestamp=tx_item['timestamp'],
+                wanted_height=tx_item.get('wanted_height', None),
+            )
             status, status_str = self.app.wallet.get_tx_status(tx_hash, tx_mined_info)
             icon = f'atlas://{KIVY_GUI_PATH}/theming/atlas/light/' + TX_ICONS[status]
             message = tx_item['label'] or tx_hash
@@ -237,9 +240,12 @@ class SendScreen(CScreen, Logger):
     def set_lnurl6(self, lnurl: str):
         url = decode_lnurl(lnurl)
         domain = urlparse(url).netloc
-        # FIXME network request blocking GUI thread:
-        lnurl_data = Network.run_from_another_thread(request_lnurl(url))
-        if not lnurl_data:
+        try:
+            # FIXME network request blocking GUI thread:
+            lnurl_data = Network.run_from_another_thread(request_lnurl(url))
+        except LNURLError as e:
+            self.app.show_error(f"LNURL request encountered error: {e}")
+            self.do_clear()
             return
         self.lnurl_data = lnurl_data
         self.address = "invoice from lnurl"
@@ -272,7 +278,7 @@ class SendScreen(CScreen, Logger):
         status = self.app.wallet.get_invoice_status(item)
         status_str = item.get_status_str(status)
         is_lightning = item.is_lightning()
-        key = self.app.wallet.get_key_for_outgoing_invoice(item)
+        key = item.get_id()
         if is_lightning:
             address = item.rhash
             if self.app.wallet.lnworker:
@@ -331,25 +337,30 @@ class SendScreen(CScreen, Logger):
             self.app.show_error(_('Please enter an amount'))
             return
         if self.is_max:
-            amount = '!'
+            amount_sat = '!'
         else:
             try:
-                amount = self.app.get_amount(self.amount)
-            except:
+                amount_sat = self.app.get_amount(self.amount)
+            except Exception:
                 self.app.show_error(_('Invalid amount') + ':\n' + self.amount)
                 return
         message = self.message
         try:
             if self.is_lightning:
-                return Invoice.from_bech32(address)
-            else:  # on-chain
+                assert type(amount_sat) is int
+                invoice = Invoice.from_bech32(address)
+                if invoice.amount_msat is None:
+                    invoice.amount_msat = int(amount_sat * 1000)
+                return invoice
+            else:
+                # on-chain
                 if self.payment_request:
                     outputs = self.payment_request.get_outputs()
                 else:
                     if not bitcoin.is_address(address):
                         self.app.show_error(_('Invalid Bitcoin Address') + ':\n' + address)
                         return
-                    outputs = [PartialTxOutput.from_address_and_value(address, amount)]
+                    outputs = [PartialTxOutput.from_address_and_value(address, amount_sat)]
                 return self.app.wallet.create_invoice(
                     outputs=outputs,
                     message=message,
@@ -373,7 +384,7 @@ class SendScreen(CScreen, Logger):
         assert self.lnurl_data
         try:
             amount = self.app.get_amount(self.amount)
-        except:
+        except Exception:
             self.app.show_error(_('Invalid amount') + ':\n' + self.amount)
             return
         if not (self.lnurl_data.min_sendable_sat <= amount <= self.lnurl_data.max_sendable_sat):
@@ -430,7 +441,7 @@ class SendScreen(CScreen, Logger):
         outputs = invoice.outputs
         amount = sum(map(lambda x: x.value, outputs)) if not any(parse_max_spend(x.value) for x in outputs) else '!'
         coins = self.app.wallet.get_spendable_coins(None)
-        make_tx = lambda rbf: self.app.wallet.make_unsigned_transaction(coins=coins, outputs=outputs, rbf=rbf)
+        make_tx = lambda: self.app.wallet.make_unsigned_transaction(coins=coins, outputs=outputs)
         on_pay = lambda tx: self.app.protected(_('Send payment?'), self.send_tx, (tx, invoice))
         d = ConfirmTxDialog(self.app, amount=amount, make_tx=make_tx, on_pay=on_pay)
         d.open()
@@ -481,7 +492,7 @@ class ReceiveScreen(CScreen):
         self.address = addr
 
     def on_address(self, addr):
-        req = self.app.wallet.get_request(addr)
+        req = self.app.wallet.get_request_by_addr(addr)
         self.status = ''
         if req:
             self.message = req.get('memo', '')
@@ -508,6 +519,7 @@ class ReceiveScreen(CScreen):
         if amount_sat and amount_sat < self.app.wallet.dust_threshold():
             self.address = ''
             if not self.app.wallet.has_lightning():
+                self.app.show_info(_('Amount too small to be received onchain'))
                 return
         else:
             addr = self.address or self.app.wallet.get_unused_address()
@@ -527,16 +539,16 @@ class ReceiveScreen(CScreen):
         self.update()
         self.app.show_request(key)
 
-    def get_card(self, req: Invoice) -> Dict[str, Any]:
+    def get_card(self, req: Request) -> Dict[str, Any]:
         is_lightning = req.is_lightning()
         if not is_lightning:
             address = req.get_address()
         else:
-            address = req.lightning_invoice
-        key = self.app.wallet.get_key_for_receive_request(req)
+            address = self.app.wallet.get_bolt11_invoice(req)
+        key = req.get_id()
         amount = req.get_amount_sat()
         description = req.message
-        status = self.app.wallet.get_request_status(key)
+        status = self.app.wallet.get_invoice_status(req)
         status_str = req.get_status_str(status)
         ci = {}
         ci['screen'] = self
@@ -562,7 +574,7 @@ class ReceiveScreen(CScreen):
         data = payments_container.data
         for item in data:
             if item['key'] == key:
-                status = self.app.wallet.get_request_status(key)
+                status = self.app.wallet.get_invoice_status(request)
                 status_str = request.get_status_str(status)
                 item['status'] = status
                 item['status_str'] = status_str
